//---konfiguracja wyświetlacza-------------------------
#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

//---funkcje wyprowadzeń-------------------------------
int button1_pin = 3;
int button2_pin = 2;
int button3_pin = 1;
int button4_pin = 0;
int output1_pin = 10;

//---zmienne-------------------------------------------
int button;
int refresh;
int autorefresh;

//---zmienne - komunikacja-----------------------------
String gsmBuffer;
int gsmComplete;
int command_delay;
int gsm_step;
int pos;
int end_pos;
String command;
int param;

//---zmienne - siec------------------------------------
String string_siec;
int sygnal;

//---zmienne- sms-------------------------------------- 
String string_param;
String string_number;
String string_sms;

//---setup---------------------------------------------
void setup() {
  //początkowy stan wyjścia - stan niski LOW
  digitalWrite(output1_pin, LOW); 
  
  //konfigurowanie wyjść
  pinMode(button1_pin, INPUT_PULLUP);
  pinMode(button2_pin, INPUT_PULLUP);
  pinMode(button3_pin, INPUT_PULLUP);
  pinMode(button4_pin, INPUT_PULLUP);
  pinMode(output1_pin, OUTPUT);
  
  //konfigurowanie wyświetlacza
  lcd.begin(16,2);
  lcd.print("Hello");

  //konfigurowanie interfejsu szeregowego UART1
  //do komunikacji z modułem GSM
  Serial.begin(19200);
  Serial1.begin(19200);

  //konfigurowanie interfejsu szeregowego UART0
  //do komunikacji przez USB 
  Serial.println("Hello");

  //początkowe wartości zmiennych
  string_siec = "Czekaj...";
  sygnal = 0;
  gsmComplete = 11;

  //czas dla wystartowania modułu GSM
  command_delay = -600;
  delay(1000);
}

//---main----------------------------------------------
void loop() {
  
  //---------------------------------------------------
  //odliczanie czasu do następnej komendy
  if (command_delay <= 300) command_delay++;
  
  //czas miną - następna komenda
  if (command_delay > 300) {
    command_delay = 0;  
    gsmBuffer = "";
    
    //info
    Serial.print("\n\rGSM step:"); 
    Serial.print(gsm_step); 
    
    //wybór komendy dla modułu GSM
    switch (gsm_step) {
      case 0: {
        //wyłącza echo
        Serial1.print("ATE0\r");
      } break;
      case 1: {
        //zezwala na logowanie do sieci GSM
        Serial1.print("AT+CREG=1\r");
      } break;
      case 2: {
        //sprawdza stan zalogowania do sieci GSM
        Serial1.print("AT+CREG?\r");      
      } break;
      case 3: {
        //pobiera z SIM nazwę operatora
        Serial1.print("AT+COPS?\r");      
      } break;
      case 4: {
        //określa tekstowy tryb odczytu SMS
        Serial1.print("AT+CMGF=1\r");
      } break;
      case 5: {
        //sprawdza siłę sygnału sieci GSM
        Serial1.print("AT+CSQ\r");      
      } break;      
      case 6: {
        //odczytuje SMS z pozycji 1
        Serial1.print("AT+CMGR=1\r");      
      } break;
      case 7: {
        //odpisuje sms na numer który przysłał wiadomość
        Serial1.print("AT+CMGS=\"");
        Serial1.print(string_number);
        Serial1.print("\"\r");
        delay(300);
        Serial1.print("OK");      
        Serial1.write(0x1A);
      } break;
      case 8: {
        //kasuje SMS z pozycji 1
        Serial1.print("AT+CMGD=1\r"); 
      } break;          
    }
  }
  
  //---------------------------------------------------
  //odpieranie z UART
  serialReceive();
  gsmReceive();

  //czeka na odebranie pakietu danych
  //pakiet konczy sie przerwą w odbieraniu danych
  if (gsmComplete <= 10) gsmComplete++;  
  if (gsmComplete == 10) {

    //wybiera akcję w zalezności od komendy
    switch (gsm_step) {
      
      case 0: {                   //ATE0
          OkGet();
          if (command == "OK") gsm_step++;           
          command_delay = 200;
      } break;
      
      case 1: {                   //AT+CREG=1\r"
          OkGet();
          gsm_step++;           
          command_delay = 200;
      } break;    
      
      case 2: {                   //AT+CREG?
        CommandGet();
        ParamGet(2);
        if (param == 1) {
          gsm_step++;
        }
      } break;
      
      case 3: {                   //AT+COPS?
         CommandGet();
         StringParamGet(3);
         string_siec = string_param;
         gsm_step++;      
      } break;
      
      case 4: {                   //AT+CMGF=1
          OkGet();
          gsm_step++;           
          command_delay = 200;
     } break;

      case 5: {                   //AT+CSQ
          CommandGet();
          if (command == "CSQ") {
            ParamGet(1);
            sygnal = param;
          }
          gsm_step++;
      } break;
      
      case 6: {                   //AT+CMGR=1
          CommandGet();
          
          //jeśli odczytano SMS 
          if (command == "CMGR") {         
            //nastepny etap - kasowanie SMS
            gsm_step = 8;            
            
            //jeśli SMS zawiera treść
            if (StringSmsGet() > 0) { 
             
              //określa numer nadawcy
              StringParamGet(2);             
              string_number = string_param; 
              
              //info
              Serial.print("\n\rNew sms:"); 
              Serial.print(string_sms); 
              Serial.print("\n\rSender:"); 
              Serial.print(string_param); 

              //jesli polecenie załaczenia 
              if (string_sms == "Out on") {
                digitalWrite(output1_pin, LOW);
                lcd.setCursor(0, 0);
                lcd.print(" OUTPUT ON      ");
                autorefresh = 0;
                //nastepny etap - wysyłanie potw.
                gsm_step = 7;
              }
              
              //jesli polecenie wyłączenia
              if (string_sms == "Out off") {
                digitalWrite(output1_pin, HIGH);
                lcd.setCursor(0, 0);
                lcd.print(" OUTPUT OFF      ");
                autorefresh = 0;
                //nastepny etap - wysyłanie potw.
                gsm_step = 7;            
              }

              //jesli polecenie restartu
              if (string_sms == "Out res") {
                digitalWrite(output1_pin, HIGH);
                lcd.setCursor(0, 0);
                lcd.print(" OUTPUT RESTART ");
                delay(2000);
                digitalWrite(output1_pin, LOW);
                autorefresh = 0;
                //nastepny etap - wysyłanie potw.
                gsm_step = 7;
              }
            }

          //jesli nie ma SMS
          } else {
            OkGet();
            //wróć do sprawdzania siły sygnału GSM
            gsm_step = 5;
          }
      } break;
      
      case 7: {                   //"AT+CMGS=
          CommandGet();
          if (command == "CMGS") gsm_step++;           
      } break; 
      
      case 8: {                   //AT+CMGD=1
          OkGet();
          gsm_step = 5;           
      } break; 
           
    }
    //na koniec wyczyść bufor odebranych danych
    gsmBuffer = "";
  }

  //sprawdzenie stanu przycisków
  button = ButtonCheck();
  if (button > 0) {
    ButtonAction();
    button = 0;
    refresh = 1;      
  }

  //automatyczne odświerzanie treści wyświetlacza
  autorefresh++;
  if (autorefresh > 100) {
    autorefresh = 0;
    refresh = 1;
  }

  //wyświetlanie
  if (refresh > 0) {
    MainWindow();
    refresh = 0;
  }

  //przerwa pętli głównej  
  delay(20);
}

//--------------------------------------------------
//odbieranie danych z USB
void serialReceive()
{
  while (Serial.available()) {          //jeśli coś odebrane
    char inData = (char)Serial.read();  //odbierz dane
    Serial1.write(inData);
    command_delay = 0;
  }
}

//--------------------------------------------------
//odbieranie danych z GSM
void gsmReceive()
{
  while (Serial1.available()) {          //jeśli coś odebrane
    char inData = (char)Serial1.read();  //odbierz dane
    gsmBuffer += inData;                 //i umieść w buforze
    gsmComplete = 0;                     //pakiet w trakcie
    command_delay = 0;                   //zaczekaj z komendą
  }
}

//------------------------------------------------------
//uzyskuje nazwę komendy z odebranego pakietu
int CommandGet(void) {
    command = "---";
    pos = gsmBuffer.indexOf("+");
    if (pos > 0) {
      pos = pos + 1;
      end_pos = gsmBuffer.indexOf(":", pos);
      command = gsmBuffer.substring(pos, end_pos);    
    }
    Serial.print("\n\rNew command:"); 
    Serial.print(command); 
}

//------------------------------------------------------
//uzyskuje potwierdzeie OK z odebranego pakietu
int OkGet(void) {
    command = "---";
    pos = gsmBuffer.indexOf("\n");
    if (pos > 0) {
      pos = pos + 1;
      end_pos = gsmBuffer.indexOf("\r", pos);
      command = gsmBuffer.substring(pos, end_pos);    
    }
    Serial.print("\n\rNew command:"); 
    Serial.print(command); 
}

//------------------------------------------------------
//uzyskuje wartość określonego parametru z pakietu
int ParamGet(int index) {
    param = 0;
    pos = gsmBuffer.indexOf(":");
    if (pos > 0) {
      pos = pos + 2;
      if (index > 1) {
        pos = gsmBuffer.indexOf(",", pos);    
        pos = pos + 1;    
      }
      if (index > 2) {
        pos = gsmBuffer.indexOf(",", pos);    
        pos = pos + 1;    
      }
      param = (gsmBuffer.charAt(pos) - '0');
      
      pos = pos + 1;
      if ((gsmBuffer.charAt(pos) >= '0') && (gsmBuffer.charAt(pos) <= '9')) {
        param *= 10;
        param += (gsmBuffer.charAt(pos) - '0');
      }
    }    
    Serial.print("\n\rNew param:"); 
    Serial.print(param); 
}

//------------------------------------------------------
//uzyskuje parametr typu String z pakietu
int StringParamGet(int index) {
    string_param = "---";
    pos = gsmBuffer.indexOf(":");
    if (pos > 0) {
      pos = pos + 2;        //skip :_
      if (index > 1) {
        pos = gsmBuffer.indexOf(",", pos);    
        pos = pos + 1;      //skip ,
      }
      if (index > 2) {
        pos = gsmBuffer.indexOf(",", pos);    
        pos = pos + 1;      //skip ,
      }
      pos = pos + 1;        //skip "
      end_pos = gsmBuffer.indexOf(34, pos);   //find second "
      string_param = gsmBuffer.substring(pos, end_pos); 
    }   
    Serial.print("\n\rNew param:"); 
    Serial.print(string_param); 
}

//------------------------------------------------------
//uzyskuje tresc SMSa z pakietu
int StringSmsGet(void) {
    pos = gsmBuffer.indexOf(":");
    if (pos > 0) {
      pos = gsmBuffer.indexOf("\n", pos);  
      pos = pos + 1;
      
      end_pos = gsmBuffer.indexOf("\r", pos);
      
      string_sms = gsmBuffer.substring(pos, end_pos);
      return 1;
    } else {
      string_sms = "";
      return -1;
    }
}

//------------------------------------------------------
//wyświetla treść wyświetlacza
int MainWindow(void) {
    lcd.setCursor(0, 0);
    lcd.print("Siec:           ");
    lcd.setCursor(6, 0);
    lcd.print(string_siec);
        
    lcd.setCursor(0, 1);    
    lcd.print("Sygnal:         ");
    lcd.setCursor(8, 1);
    lcd.print(sygnal);
    lcd.print("/31  ");        
}

//------------------------------------------------------
//spradza stan przycisków
int ButtonCheck(void) {
  int result = 0;    
  static int prev;
  static int hold;

  if (digitalRead(button1_pin) == LOW) result = 1;
  if (digitalRead(button2_pin) == LOW) result = 2;
  if (digitalRead(button3_pin) == LOW) result = 3;
  if (digitalRead(button4_pin) == LOW) result = 4;

  if (result > 0) {
    if (hold < 255) hold++;
    if (result != prev) hold = 0;
    prev = result;
    result = 0;
    if (hold == 5) result = prev;
  } else {
    hold = 0;
    prev = 0;
  }
  return result;
}

//------------------------------------------------------
//wybór reakcji na przyciśnięcie przycisku
int ButtonAction(void) {
    if (button == 1) {
      //załacza wyjście
      digitalWrite(output1_pin, LOW);
    }
    if (button == 2) {
      //wyłącza wyjście
      digitalWrite(output1_pin, HIGH);
    }
    if (button == 3) {
    }
    if (button == 4) {
    }  
}
